import os
import argparse


ALE2RTC_usage = '''
================================ ALE2RTC example commands ================================

TreeSAK ALE2RTC -h

==========================================================================================
'''


def ALE2RTC():

    pass

    # file_in     = args['i']
    # op_dir      = args['o']


# if __name__ == '__main__':
#
#     ALE2RTC_parser = argparse.ArgumentParser()
#     ALE2RTC_parser.add_argument('-i',     required=True,                   help='the file "out" generated by MCMCTree')
#     ALE2RTC_parser.add_argument('-o',     required=True,                   help='output directory, which will be the input to the pRTC module (-rrtc)')
#     args = vars(ALE2RTC_parser.parse_args())
#     ALE2RTC(args)

ALE2RTC()
